#!/usr/bin/env node

const colors = require('colors');
const path = require('path');
const program = require('commander');
const inquirer = require('inquirer');
const exec = require('child_process').exec;

const tool = require('./src');

/*/
1、是否单独设置相对路径 -p 
2、是否重新命名文件名 -n name/template
3、是否有模板类别 -t type/component 
/*/
const config = {
    type: 'c',
    filename: 'template',
    name: 'Template',
    path: process.cwd(),
    init: false,
    map:false
}

program
    .version('0.0.1')
    .option(
        '-t, --type [value]',
        `choose type，
        c component, 
        m module, 
        d directive, 
        s mobk,
        p pipe, n ngrx, 
        a ngrx/action, 
        spec-c component-spec with input ouput provider event, 
        spec-s service-spec, 
        spec-d directive-spec, 
        spec-p pipe-spec, default component`,
        'c'
    )
    .option('-n, --file [value]', 'file name, default template', 'template')
    .option('-p, --path [value]', 'need relative file path', '')
    .usage('<keywords>')
    .parse(process.argv);

console.log('ng4创建模板工具 start'.green);
console.log('');

if (!program.args.length) {
    console.log('命令错误，暂时只支持init、create、test命令'.underline.red);
    program.help();
} else {
    if (program.args[0] === 'init') {
        // 初始化项目
        config.init = true;
        config.mpa = false;
        if (program.path) {
            config.path = path.join(config.path, program.path);
        }
        tool(config);
        return null;
    }
    if (program.args[0] === 'mpa') {
        // 初始化项目
        config.init = false;
        config.mpa = true;
        if (program.path) {
            config.path = path.join(config.path, program.path);
        }
        tool(config);
        return null;
    }
    if (program.args[0] === 'create') {
        config.init = false;
        config.mpa = false;
        if (program.type) {
            config.type = program.type;
        }
        if (program.file) {
            config.filename = program.file;
            config.name = program.file.toLowerCase().replace(/( |^)[a-z]/g, (L) => L.toUpperCase());
        }
        if (program.path) {
            config.path = path.join(config.path, program.path);
        }
        tool(config);
        return null;
    }
    if (program.args[0] === 'check') {
        // 创建检查组件模板
        if(!program.args[1]){
            console.log('请输入组件目录名'.underline.red);
            return null;
        }
        
        return null
    }
    if (program.path) {
        config.path = path.join(config.path, program.path);
    }
    //兼容ng-cli 命令
    inquirer.prompt([{
        type: 'input',
        name: 'isNg',
        message: '暂不支持该命令，是否尝试执行'.yellow + ' ` ng '.red + (program.args.join(' ') + ' `').red + ' ？(Y/N)'.yellow,
        validate: function (input) {
            if (input === 'N' || input === 'Y' || input === 'n' || input === 'y' || !input) {
                return true;
            }
            return '请输入Y/y或N/n';
        }
    }]).then(function (answers) {
        if (answers.isNg === 'y' || answers.isNg === 'Y') {
            console.log('');
            console.log(('命令执行目录：' + config.path).underline.yellow);
            console.log('');

            const child = exec('ng ' + program.args.join(' '), {
                cwd: config.path
            },function(error, stdout, stderr){
                if(error){
                    console.log(colors.red('Error message!'));
                    console.log(colors.red(error));
                }
            });
            child.stdout.on('data', function (data) {
                console.log(colors.green(data));
            });
            child.stderr.on('data', function (data) {
                console.log(colors.red.underline(data));
            });
            child.on('close', function (code) {
                if(code===0){
                    console.log(colors.yellow.underline('命令执行完成！'));
                }else{
                    console.log(colors.red('Error code：'+ code+ ' !'));
                }
               
            });

        } else {
            console.log('');
            console.log('命令已经取消'.underline.yellow);
        }
    })

    //console.log(('暂不支持该命令，将尝试执行ng ' + program.args.join(' ')).underline.red);

}