const express = require('express')
const webpack = require('webpack')
const path = require('path');
const colors = require('colors');
const WebpackHotMiddleware = require('webpack-hot-middleware');
const webpackConfig = require('./webpack/webpack.dev');
const app = express();

const PORT = process.env.PORT || 4000;
/* webpack 编译器 */
const compiler = webpack(webpackConfig);
/* webpack-dev-server中间件 */
const devMiddleware = require('webpack-dev-middleware')(compiler, {
    publicPath: webpackConfig.output.publicPath,
    stats: {
        colors: true,
        chunks: true
    },
    progress: true,
    inline: true,
    hot: true
});

app.use(devMiddleware);
app.use(WebpackHotMiddleware(compiler));

/* 路由 */
app.get('/:viewname?', (req, res, next) => {
    const viewname = req.params.viewname ? req.params.viewname + '.html' : 'index.html';
    const filepath = path.join(compiler.outputPath, viewname);
    if(filepath.indexOf('.map.')>-1){
        return null;
    }
    compiler.outputFileSystem.readFile(filepath, function (err, result) {
        if (err) {
            res.set('content-type', 'text/html');
            res.send('<h1>查找 <span color="red">'+viewname+'</span> 页面错误</h1><div>'+err+'</div>');
            res.end();
            return next(colors.red(err));            
        }
        res.set('content-type', 'text/html');
        res.send(result);
        res.end();
    });
});
module.exports = app.listen(PORT, function (err) {
    if (err) {
        // do something
        return;
    }

    console.log(colors.green('Listening at http://localhost:'+PORT + '\n'))
})