const helpers = require('../helpers');

const CommonsChunkPlugin = require('webpack/lib/optimize/CommonsChunkPlugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const HtmlElementsPlugin = require('../html-elements-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const InlineManifestWebpackPlugin = require('inline-manifest-webpack-plugin');
const LoaderOptionsPlugin = require('webpack/lib/LoaderOptionsPlugin');
const ScriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin');
const ngcWebpack = require('ngc-webpack');
const buildUtils = require('../build-utils');

/*
 * Webpack 常量配置
 */
module.exports = function (options, pagename) {
	const isProd = options.env === 'product';
	const METADATA = Object.assign({}, buildUtils.DEFAULT_METADATA, options.metadata || {});
	const ngcWebpackConfig = buildUtils.ngcWebpackSetup(isProd, METADATA);
	Object.assign(ngcWebpackConfig.plugin, {
		tsConfigPath: METADATA.tsConfigPath,
		mainPath: './src/pages/' + pagename + '/main.browser.ts'
	});

	return {
		resolve: {
			mainFields: ['browser', 'module', 'main'],
			extensions: ['.ts', '.js', '.json'],
			modules: [helpers.root('src'), helpers.root('node_modules')],
		},
		module: {

			rules: [
				...ngcWebpackConfig.loaders,
				{
					test: /\.css$/,
					use: ['to-string-loader', 'css-loader'],
					exclude: [helpers.root('src', 'styles')]
				},
				{
					test: /\.scss$/,
					use: ['to-string-loader', 'css-loader', 'sass-loader'],
					exclude: [helpers.root('src', 'styles')]
				},
				{
					test: /\.html$/,
					use: 'html-withimg-loader',
					exclude: [helpers.root('src/index.html')]
				},
				{
					test: /\.(jpg|png|gif|svg)$/,
					use: [{
						loader: 'file-loader',
						options: {
							name:'dist/common/assets/images/[name].[hash:8].[ext]'
						}
					}]
				},

				{
					test: /\.(eot|woff2?|ttf)([\?]?.*)$/,
					use: [{
						loader: 'file-loader',
						options: {
							name:'dist/common/assets/fonts/[name].[hash:8].[ext]'
						}
					}]
				}
			],
		},
		plugins: [
			new CopyWebpackPlugin([{
				from: 'src/assets',
				to: 'dist/common/assets'
			}]),
			new HtmlElementsPlugin({
				headTags: require('../header-config.common')
			}),

			new LoaderOptionsPlugin({}),

			new ngcWebpack.NgcWebpackPlugin(ngcWebpackConfig.plugin),
			new InlineManifestWebpackPlugin(),
		],
		node: {
			global: true,
			crypto: 'empty',
			process: true,
			module: false,
			clearImmediate: false,
			setImmediate: false
		}

	};
}