const glob = require('glob');
const webpack = require('webpack');
const CommonsChunkPlugin = require('webpack/lib/optimize/CommonsChunkPlugin');
const DefinePlugin = require('webpack/lib/DefinePlugin');
const webpackMerge = require('webpack-merge');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ngToolWebpack = require('@ngtools/webpack');
const helpers = require('../helpers');
const commonWebpackConfig = require('./dev.common')('dev');
const hotMiddlewareScript = 'webpack-hot-middleware/client?reload=true'

function getEntries(globPath) {
    const files = glob.sync(globPath),
        entries = {};
    files.forEach(function (filepath) {
        // 取倒数第二层(view下面的文件夹)做包名
        const split = filepath.split('/');
        const name = split[split.length - 2];

        entries[name] = ['./' + filepath, hotMiddlewareScript];
    });
    return entries;
}

const entries = getEntries('src/pages/**/main.browser.ts');
const chunks = Object.keys(entries);
const plugins = [
    new DefinePlugin({
        'ENV': JSON.stringify('local'),
        'process.env': {
            'ENV': JSON.stringify('local'),
            'API': JSON.stringify('http://test-iappweb.jpushoa.com/v1/')
        }
    }),
    new CommonsChunkPlugin({
        name: 'polyfills',
        chunks: ['polyfills']
    }),
    new CommonsChunkPlugin({
        name: 'vendor',
        chunks: chunks,
        minChunks: chunks.length
    }),
    new webpack.HotModuleReplacementPlugin()
];
chunks.forEach((item, index) => {
    const confHtml = {
        filename: item + '.html',
        template: 'src/index.html',
        inject: true,
        chunks: ['polyfills', 'vendor', item],
         chunksSortMode: function (a, b) {
            const entryPoints = ["polyfills", "vendor", item];
            return entryPoints.indexOf(a.names[0]) - entryPoints.indexOf(b.names[0]);
        }
    }
    plugins.push(new HtmlWebpackPlugin(confHtml));
})
entries['polyfills'] = ['./src/polyfills.browser.ts', hotMiddlewareScript];

module.exports = webpackMerge(commonWebpackConfig, {
    entry: entries,
    module: {
        rules: [{
                test: /\.css$/,
                use: ['style-loader', 'css-loader'],
                include: [helpers.root('src', 'styles')]
            },
            {
                test: /\.scss$/,
                use: ['style-loader', 'css-loader', 'sass-loader'],
                include: [helpers.root('src', 'styles')]
            }
        ]
    },
    plugins: plugins
})